package com.zswl.dataservicestarter.service;

import com.zswl.dataservicestarter.dao.DataServiceGoodsDao;
import com.zswl.dataservicestarter.dao.DataServiceOrderInfoDao;
import com.zswl.dataservicestarter.dao.DataServicePaymentInfoDao;
import com.zswl.dataservicestarter.domain.DataServiceGoods;
import com.zswl.dataservicestarter.domain.DataServiceOrderInfo;
import com.zswl.dataservicestarter.domain.DataServicePaymentInfo;
import com.zswl.dataservicestarter.model.DataServiceGoodsModel;
import com.zswl.dataservicestarter.model.DataServiceOrderInfoModel;
import com.zswl.dataservicestarter.model.DataServicePaymentInfoModel;
import com.zswl.dataservicestarter.model.SyncDataModel;
import com.zswl.dataservicestarter.type.DataServiceOrderType;
import com.zswl.dataservicestarter.type.PlatFormNewDataEvent;
import com.zswl.dataservicestarter.type.ShopNewDataEvent;
import com.zswl.dataservicestarter.utils.CommonUtil;
import com.zswl.dataservicestarter.utils.result.ResultContent;
import lombok.SneakyThrows;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 订单信息 管理
 *
 * @author TRX
 * @date 2024/3/25
 */
@Service
public class DataServiceOrderInfoService {

    @Autowired
    DataServiceOrderInfoDao orderInfoDao;

    @Autowired
    DataServiceGoodsDao dataServiceGoodsDao;

    @Autowired
    DataServicePaymentInfoDao paymentInfoDao;

    @Autowired
    ApplicationContext applicationContext;

    /**
     * 从平台同步订单信息到 数据服务
     *
     * @param model
     * @return
     */
    @Transactional
    public ResultContent saveOrderInfoFromPlatform(DataServiceOrderInfoModel model) {
        // 验证数据完整性
        if (StringUtils.isEmpty(model.getEpId())) {
            return ResultContent.buildFail("商铺ID不能为空");
        }
        if (StringUtils.isEmpty(model.getOrderNo())) {
            return ResultContent.buildFail("订单号不能为空");
        }
        if (model.getShopInfo() == null) {
            return ResultContent.buildFail("店铺信息为空");
        }
        if (CommonUtil.bigDecimalIsEmpty(model.getTotalAmount())) {
            return ResultContent.buildFail("商品总价格不能为空");
        }
        String orderNo = model.getOrderNo();
        DataServiceOrderInfo tempOrderInfo = orderInfoDao.findTopByOrderNo(orderNo);
        if (ObjectUtils.isNotEmpty(tempOrderInfo)) {
//            return ResultContent.buildFail(String.format("%s 订单号已存在", model.getOrderNo()));
        }
        if (CommonUtil.bigDecimalIsEmpty(model.getDiscountAmount())) {
            return ResultContent.buildFail("优惠价格不能为空");
        }
        if (CommonUtil.bigDecimalIsEmpty(model.getPayAmount())) {
            return ResultContent.buildFail("实际支付价格不能为空");
        }

        // 订单管理的商品信息
        List<DataServiceGoodsModel> goodsList = model.getGoodsList();
        if (ObjectUtils.isEmpty(goodsList)) {
            return ResultContent.buildFail("商品不能为空");
        }
        List<DataServiceGoods> goodsList1 = new ArrayList<>();
        for (DataServiceGoodsModel goodsModel : goodsList) {
            DataServiceGoods goods = new DataServiceGoods();
            if (StringUtils.isEmpty(goodsModel.getGoodsId())) {
                return ResultContent.buildFail(String.format("商品id不能为空"));
            }
            if (goodsModel.getGoodsInfo() == null) {
                return ResultContent.buildFail(String.format("商品信息不能为空"));
            }

            BeanUtils.copyProperties(goodsModel, goods);
            goods.setOrderNo(orderNo);

            goodsList1.add(goods);
        }

        // 支付信息
        DataServicePaymentInfoModel paymentInfoModel = model.getPayment();
        if (StringUtils.isEmpty(paymentInfoModel.getPaymentTime())) {
            return ResultContent.buildFail("支付时间为空");
        }
        DataServicePaymentInfo paymentInfo = new DataServicePaymentInfo();
        BeanUtils.copyProperties(paymentInfoModel, paymentInfo);
        paymentInfo.setOrderNo(orderNo);

        dataServiceGoodsDao.saveAll(goodsList1);
        paymentInfoDao.save(paymentInfo);

        // 保存响应数据
        DataServiceOrderInfo orderInfo = new DataServiceOrderInfo();
        BeanUtils.copyProperties(model, orderInfo);
        orderInfo.setGoodsList(goodsList1);
        orderInfo.setPayment(paymentInfo);
        orderInfo.setOrderType(DataServiceOrderType.Platform);
        orderInfo.setIsSync2Platform(Boolean.FALSE);
        orderInfo.setIsSync2Shop(Boolean.FALSE);
        int hasCode = CommonUtil.getEntityHashCode(orderInfo);
        orderInfo.setHashCode(hasCode);

        orderInfoDao.save(orderInfo);

        // 发送需要商铺同步数据的事件
        this.sendShopEventData();
        return ResultContent.buildSuccess(hasCode);
    }

    /**
     * 得到未同步到平台的数据  同步成功后调用 markSynced2Platform
     *
     * @return
     */
    public ResultContent<SyncDataModel> getTop100NotSyncPlatformDatas() {
        SyncDataModel model = new SyncDataModel();
        // 查询来源是商铺的数据  未同步
        List<DataServiceOrderInfo> orderInfos = orderInfoDao.findTop100ByOrderTypeAndIsSync2ShopOrderByCreateTimeAsc(DataServiceOrderType.Shop, Boolean.FALSE);
        long totalElements = orderInfoDao.countByOrderTypeAndIsSync2Shop(DataServiceOrderType.Shop, Boolean.FALSE);
        List<DataServiceOrderInfoModel> list = new ArrayList<>();
        if (ObjectUtils.isNotEmpty(orderInfos)) {
            list = orderInfos.stream().map(this::toModel).collect(Collectors.toList());
        }
        model.setList(list);
        model.setTotalElements(totalElements);
        return ResultContent.buildSuccess(model);
    }

    /**
     * 查询未同步到渠道/商铺的数据 同步后要调用 markSynced2Shop
     *
     * @return
     */
    public ResultContent<SyncDataModel> getTop100NotSyncShopDatas() {
        SyncDataModel model = new SyncDataModel();
        // 查询来源是平台的数据 未同步到商铺
        List<DataServiceOrderInfo> orderInfos = orderInfoDao.findTop100ByOrderTypeAndIsSync2PlatformOrderByCreateTimeAsc(DataServiceOrderType.Platform, Boolean.FALSE);
        long totalElements = orderInfoDao.countByOrderTypeAndIsSync2Platform(DataServiceOrderType.Platform, Boolean.FALSE);
        List<DataServiceOrderInfoModel> list = new ArrayList<>();
        if (ObjectUtils.isNotEmpty(orderInfos)) {
            list = orderInfos.stream().map(this::toModel).collect(Collectors.toList());
        }
        model.setList(list);
        model.setTotalElements(totalElements);
        return ResultContent.buildSuccess(model);
    }

    public ResultContent sendPlatformEventData() {
        PlatFormNewDataEvent testEvent = new PlatFormNewDataEvent(this);
        applicationContext.publishEvent(testEvent);
        return ResultContent.buildSuccess();
    }

    public ResultContent sendShopEventData() {
        ShopNewDataEvent testEvent = new ShopNewDataEvent(this);
        applicationContext.publishEvent(testEvent);
        return ResultContent.buildSuccess();
    }

    @EventListener(classes = PlatFormNewDataEvent.class)
    @Async
    @SneakyThrows
    public void test(PlatFormNewDataEvent testEvent) {
        TimeUnit.SECONDS.sleep(10);
        System.out.println("消息来了");
    }

    /**
     * 根据订单号查询订单信息
     *
     * @param orderNo
     * @return
     */
    public ResultContent<DataServiceOrderInfo> getOrderInfoByOrderNo(String orderNo) {
        if (StringUtils.isEmpty(orderNo)) {
            return ResultContent.buildFail("订单号为空");
        }
        DataServiceOrderInfo orderInfo = orderInfoDao.findTopByOrderNo(orderNo);

        return ResultContent.buildSuccess(orderInfo);
    }


    /**
     * 更加HashCode验证支付信息是否存在
     *
     * @param hashCode
     * @return
     */
    public ResultContent validateDataByHashCode(int hashCode) {
        DataServiceOrderInfo orderInfo = orderInfoDao.findTopByHashCode(hashCode);
        if (ObjectUtils.isEmpty(orderInfo)) {
            return ResultContent.buildFail("对应数据未找到");
        }
        return ResultContent.buildSuccess();
    }

    /**
     * 编辑已同步到 商铺
     *
     * @param orderNos
     * @return
     */
    public ResultContent<List<String>> markSynced2Shop(List<String> orderNos) {
        List<String> syncedOrderNos = new ArrayList<>();
        if (ObjectUtils.isNotEmpty(orderNos)) {
            List<DataServiceOrderInfo> saveList = new ArrayList<>();
            List<DataServiceOrderInfo> list = orderInfoDao.findByOrderNoIn(orderNos);
            for (DataServiceOrderInfo orderInfo : list) {
                // 未同步的
                Boolean isSync2Shop = orderInfo.getIsSync2Shop();
                if (isSync2Shop == null || !isSync2Shop) {
                    orderInfo.setIsSync2Shop(Boolean.TRUE);
                    orderInfo.setSyncTime(System.currentTimeMillis());
                    saveList.add(orderInfo);
                }
            }
            if (ObjectUtils.isNotEmpty(saveList)) {
                syncedOrderNos = saveList.stream().map(it -> it.getOrderNo()).collect(Collectors.toList());
            }
        }
        return ResultContent.buildSuccess(syncedOrderNos);
    }

    /**
     * 编辑已同步到 商铺
     *
     * @param orderNos
     * @return
     */
    public ResultContent<List<String>> markSynced2Platform(List<String> orderNos) {
        List<String> syncedOrderNos = new ArrayList<>();
        if (ObjectUtils.isNotEmpty(orderNos)) {
            List<DataServiceOrderInfo> saveList = new ArrayList<>();
            List<DataServiceOrderInfo> list = orderInfoDao.findByOrderNoIn(orderNos);
            for (DataServiceOrderInfo orderInfo : list) {
                // 未同步的
                Boolean isSync2Platform = orderInfo.getIsSync2Platform();
                if (isSync2Platform == null || !isSync2Platform) {
                    orderInfo.setIsSync2Platform(Boolean.TRUE);
                    orderInfo.setSyncTime(System.currentTimeMillis());
                    saveList.add(orderInfo);
                }
            }
            if (ObjectUtils.isNotEmpty(saveList)) {
                syncedOrderNos = saveList.stream().map(it -> it.getOrderNo()).collect(Collectors.toList());
            }
        }
        return ResultContent.buildSuccess(syncedOrderNos);
    }

    /**
     * 订单信息 转Model
     *
     * @param orderInfo
     * @return
     */
    public DataServiceOrderInfoModel toModel(DataServiceOrderInfo orderInfo) {
        DataServiceOrderInfoModel model = new DataServiceOrderInfoModel();
        if (ObjectUtils.isNotEmpty(orderInfo)) {
            com.zswl.dataservicestarter.utils.BeanUtils.copyProperties(orderInfo, model);
        }
        return model;
    }

}
